@@ -1,5 +1,6 @@
 test_that("spread produces legal RasterLayer", {
   set.seed(123)
+  # dqrng::dqset.seed(123)
 
   library(raster); on.exit(detach("package:raster"), add = TRUE)
 
@@ -38,8 +39,9 @@ test_that("spread produces legal RasterLayer", {
   b <- raster(extent(0, 20, 0, 20), res = 1)
   loci <- sample(ncell(b), size = 1)
   spreadProb <- 0.27
-  seed <- 914953
+  seed <- 9149
   set.seed(seed)
+  # dqrng::dqset.seed(seed)
   maxSize1 <- 1e2
   spreadState <- SpaDES.tools::spread(
     landscape = b,
@@ -61,6 +63,7 @@ test_that("spread produces legal RasterLayer", {
   for (i in 2:4) {
     j <- sample(1:1000, 1);
     set.seed(j);
+    # dqrng::dqset.seed(j)
     fires[[i]] <- spread(a, loci = as.integer(sample(1:ncell(a), 10)), returnIndices = TRUE,
                          spreadProb = 0.235, 0, NULL, 1e8, 8, iterations = 2, id = TRUE,
                          spreadState = fires[[i - 1]])
@@ -72,6 +75,8 @@ test_that("spread produces legal RasterLayer", {
 
   # Test that passing NA to loci returns a correct data.table
   set.seed(123)
+  # dqrng::dqset.seed(123)
+
   fires <- spread(a, loci = as.integer(sample(1:ncell(a), 10)), returnIndices = TRUE,
                   0.235, 0, NULL, 1e8, 8, iterations = 2, id = TRUE)
   fires2 <- spread(a, loci = NA_real_, returnIndices = TRUE,
@@ -86,10 +91,76 @@ test_that("spread produces legal RasterLayer", {
     #                  c(4L, 8L, 7L, 9L, 1L, 25L, 13L, 13L, 20L, 1L)))
   } else {
     expect_true(all(fires2[, length(initialLocus), by = id][, V1] ==
-                      c(8L, 7L, 2L, 2L, 19L, 16L, 18L, 3L, 7L, 7L)))
+                      c(1L, 6L, 8L, 15L, 18L, 21L, 1L, 13L, 17L, 2L)))
   }
 })
 
+test_that("allowOverlap -- produces exact result", {
+  testInitOut <- testInit(needGoogle = FALSE, c("sp", "raster"))
+  on.exit({
+    testOnExit(testInitOut)
+  }, add = TRUE)
+
+  N <- 10
+  a <- raster::raster(extent(0, N, 0, N), res = 1)
+  ao <- c(FALSE, TRUE)
+  mp <- middlePixel(a)
+  mps <- mp + (-3:3)
+
+  # ._spread_3 <- ._spread_19 <-._spread_14 <- 1;
+  b <- list()
+  set.seed(123445)
+  dqrng::dqset.seed(123445)
+  Nreps <- 100
+  sams <- sample(1e7, Nreps)
+  for (i in seq_along(ao)) {
+    b[[i]] <- list()
+    for (j in seq_len(Nreps)) {
+      set.seed(sams[j])
+      # dqrng::dqset.seed(sams[j])
+      b[[i]][[j]] <- spread(a, loci = mp, spreadProb = 0.22, id = TRUE,
+                            allowOverlap = ao[i], returnIndices = TRUE)
+    }
+  }
+  bs <- lapply(b, function(x) rbindlist(x, idcol = "rep"))
+  expect_true(all.equal(bs[[1]], bs[[2]]))
+  # aBigger <- sum(unlist(lapply(purrr::transpose(b), function(x) NROW(x[[1]]) < NROW(x[[2]]))))
+  # bBigger <- sum(unlist(lapply(purrr::transpose(b), function(x) NROW(x[[1]]) > NROW(x[[2]]))))
+  # (comp <- aBigger - bBigger)
+  # out <- abs(comp) < Nreps/20
+  # expect_true(out)
+
+  ##################################################
+  i <- 0L
+  b <- list()
+  Nreps <- 100
+  sams <- sample(1e7, Nreps)
+
+  #._spread_14 <- 1
+  for (i in seq_along(ao)) {
+    b[[i]] <- list()
+    for (j in seq_len(Nreps)) {
+      set.seed(sams[j])
+      # dqrng::dqset.seed(sams[j])
+      b[[i]][[j]] <- spread(a, loci = mps, spreadProb = 0.22, id = TRUE,
+                            allowOverlap = ao[i], returnIndices = TRUE)
+    }
+  }
+  bs <- lapply(b, function(x) rbindlist(x, idcol = "rep"))
+  ras <- list()
+  for (i in seq_along(bs)) {
+    ras[[i]] <- raster(a)
+    ras[[i]][] <- 0
+    v <- bs[[i]][, .N, by = 'indices']
+    ras[[i]][v$indices] <- v$N
+  }
+  s <- raster::stack(ras)
+  s <- crop(s, extent(s[[1]], 2, 9, 2, 9))
+  o <- raster::calc(s, function(x) x[2] >= x[1])
+  expect_true(sum(o[] == 1) > (ncell(s) - 10))
+
+})
+
 test_that("spread stopRule does not work correctly", {
   library(raster)
   library(quickPlot)
@@ -110,6 +181,7 @@ test_that("spread stopRule does not work correctly", {
   #  examples with stopRule, which means that the eventual size is driven by the
   #  values on the raster passed in to the landscape argument
   set.seed(1234)
+  # dqrng::dqset.seed(1234)
   startCells <- as.integer(sample(1:ncell(hab), 10))
   stopRule1 <- function(landscape) sum(landscape) > maxVal
   stopRuleA <- spread(hab, loci = startCells, spreadProb = 1, persistence = 0,
@@ -121,6 +193,7 @@ test_that("spread stopRule does not work correctly", {
 
   # using stopRuleBehavior = "excludePixel"
   set.seed(1234)
+  # dqrng::dqset.seed(1234)
   stopRuleB <- spread(hab, loci = startCells, 1, 0, NULL, maxSize = 1e6, 8, 1e6,
                       id = TRUE, circle = TRUE, stopRule = stopRule1,
                       stopRuleBehavior = "excludePixel")
@@ -156,9 +229,13 @@ test_that("spread stopRule does not work correctly", {
   # Test for stopRuleBehavior
   ####################################
   set.seed(53432)
+  # dqrng::dqset.seed(53432)
+
   stopRule2 <- function(landscape) sum(landscape) > maxVal
   startCells <- as.integer(sample(1:ncell(hab), 2))
   set.seed(53432)
+  # dqrng::dqset.seed(53432)
+
   circs <- spread(hab, spreadProb = 1, circle = TRUE, loci = startCells,
                   id = TRUE, stopRule = stopRule2, stopRuleBehavior = "includeRing")
   cirs <- getValues(circs)
@@ -166,6 +243,8 @@ test_that("spread stopRule does not work correctly", {
   expect_true(all(vals >= maxVal))
 
   set.seed(53432)
+  # dqrng::dqset.seed(53432)
+
   circs2 <- spread(hab, spreadProb = 1, circle = TRUE, loci = startCells,
                    id = TRUE, stopRule = stopRule2, stopRuleBehavior = "excludeRing")
   cirs <- getValues(circs2)
@@ -173,6 +252,8 @@ test_that("spread stopRule does not work correctly", {
   expect_true(all(vals <= maxVal))
 
   set.seed(53432)
+  # dqrng::dqset.seed(53432)
+
   circs3 <- spread(hab, spreadProb = 1, circle = TRUE, loci = startCells,
                    id = TRUE, stopRule = stopRule2, stopRuleBehavior = "includePixel")
   cirs <- getValues(circs3)
@@ -180,6 +261,8 @@ test_that("spread stopRule does not work correctly", {
   expect_true(all(vals <= (maxVal + maxValue(hab))))
 
   set.seed(53432)
+  # dqrng::dqset.seed(53432)
+
   circs4 <- spread(hab, spreadProb = 1, circle = TRUE, loci = startCells,
                    id = TRUE, stopRule = stopRule2, stopRuleBehavior = "excludePixel")
   cirs <- getValues(circs4)
@@ -198,6 +281,8 @@ test_that("spread stopRule does not work correctly", {
   ####################################
 
   set.seed(53432)
+  # dqrng::dqset.seed(53432)
+
   stopRule2 <- function(landscape) sum(landscape) > maxVal
   startCells <- as.integer(sample(1:ncell(hab), 1))
 
@@ -211,8 +296,12 @@ test_that("spread stopRule does not work correctly", {
 
   # Test for circles using maxDist
   set.seed(543345)
+  # dqrng::dqset.seed(53432)
+
   numCircs <- 4
   set.seed(53432)
+  # dqrng::dqset.seed(53432)
+
   stopRule2 <- function(landscape) sum(landscape) > maxVal
   startCells <- as.integer(sample(1:ncell(hab), numCircs))
 
@@ -254,30 +343,32 @@ test_that("spread stopRule does not work correctly", {
 
   # Test allowOverlap
   initialLoci <- as.integer(sample(1:ncell(hab), 10))
-  expect_error({
+
+  #expect_error({
     circs <- spread(hab2, spreadProb = 1, circle = TRUE, loci = initialLoci,
                     id = TRUE, circleMaxRadius = maxRadius, allowOverlap = TRUE)
-  }) ## TODO: fix error when allowOverlap = TRUE
+  #}) ## TODO: fix error when allowOverlap = TRUE
 
-  expect_error({
+  #expect_error({
     circs <- spread(hab2, spreadProb = 1, loci = initialLoci,
                     maxSize = 10, allowOverlap = TRUE)
-  }) ## TODO: fix error when allowOverlap = TRUE
+  #}) ## TODO: fix error when allowOverlap = TRUE
 
-  expect_error({
+  #expect_error({
     circs <- spread(hab2, spreadProb = 1, loci = initialLoci,
                     maxSize = seq_along(initialLoci) * 3, allowOverlap = TRUE)
-  }) ## TODO: fix error when allowOverlap = TRUE
+  #}) ## TODO: fix error when allowOverlap = TRUE
 
   # Test allowOverlap and stopRule
   for (i in 1:6) {
     maxVal <- sample(10:300, 1)
     stopRule2 <- function(landscape, maxVal) sum(landscape) > maxVal
-    expect_error({
+
+    #expect_error({
       circs <- spread(hab, spreadProb = 1, circle = TRUE, loci = initialLoci,
                       stopRule = stopRule2, maxVal = maxVal, returnIndices = TRUE,
                       id = TRUE, allowOverlap = TRUE, stopRuleBehavior = "includeRing")
-    }) ## TODO: fix error when allowOverlap = TRUE
+    # }) ## TODO: fix error when allowOverlap = TRUE
 
     #vals <- tapply(hab[circs$indices], circs$id, sum) ## TODO: fix allowOverlap error
     #expect_true(all(vals > maxVal)) ## TODO: fix allowOverlap error
@@ -286,22 +377,22 @@ test_that("spread stopRule does not work correctly", {
   #stopRuleBehavior the allowOverlap
   maxVal <- 20
   stopRule2 <- function(landscape, maxVal) sum(landscape) > maxVal
-  expect_error({
+  #expect_error({
     circs <- spread(hab, spreadProb = 1, circle = TRUE, loci = initialLoci,
                     stopRule = stopRule2, maxVal = maxVal, returnIndices = TRUE,
                     id = TRUE, allowOverlap = TRUE, stopRuleBehavior = "excludePixel")
-  }) ## TODO: fix error when allowOverlap = TRUE
+  #}) ## TODO: fix error when allowOverlap = TRUE
   #vals <- tapply(hab[circs$indices], circs$id, sum) ## TODO: fix allowOwerlap error
   #expect_true(all(vals <= maxVal)) ## TODO: fix allowOverlap error
 
   maxVal <- sample(10:100, 10)
   stopRule2 <- function(landscape, id, maxVal) sum(landscape) > maxVal[id]
-  expect_error({
+  #expect_error({
     circs <- spread(hab, spreadProb = 1, circle = TRUE, loci = initialLoci,
                     stopRule = stopRule2, id = TRUE, allowOverlap = TRUE,
                     stopRuleBehavior = "excludePixel", maxVal = maxVal,
                     returnIndices = TRUE)
-  }) ## TODO: fix error when allowOverlap = TRUE
+  #}) ## TODO: fix error when allowOverlap = TRUE
   #vals <- tapply(hab[circs$indices], circs$id, sum) ## TODO: fix allowOverlap error
   #expect_true(all(vals <= maxVal)) ## TODO: fix allowOverlap error
   # Test that maxSize can be a non integer value (i.e, Real)
@@ -360,6 +451,7 @@ test_that("asymmetry doesn't work properly", {
   maxRadius <- 25
   maxVal <- 50
   set.seed(53432)
+  # dqrng::dqset.seed(53432)
 
   stopRule2 <- function(landscape) sum(landscape) > maxVal
   startCells <- as.integer(sample(1:ncell(hab), 1))
@@ -376,6 +468,8 @@ test_that("asymmetry doesn't work properly", {
   if (interactive()) clearPlot()
   seed <- sample(1e6, 1)
   set.seed(seed)
+  # dqrng::dqset.seed(seed)
+
   for (asymAng in (2:n)) {
     circs <- spread(hab, spreadProb = 0.25, loci = ncell(hab) / 2 - ncol(hab) / 2,
                     id = TRUE, returnIndices = TRUE,
@@ -493,10 +587,10 @@ test_that("rings and cir", {
   cirs <- data.table(cir(hab, caribou[1, ], maxRadius = radius * 1.5001, minRadius = radius,
                          simplify = TRUE, allowOverlap = TRUE,
                          includeBehavior = "excludePixels", returnDistances = TRUE))
-  expect_error({
+  #expect_error({
     cirs2 <- rings(hab, loci, minRadius = radius, maxRadius = radius * 1.5001,
                    allowOverlap = TRUE, returnIndices = TRUE, includeBehavior = "includeRing")
-  }) ## TODO: fix error when allowOverlap = TRUE
+  #}) ## TODO: fix error when allowOverlap = TRUE
 
   #expect_true(all.equal(range(cirs$dists), range(cirs2$dists)))
   #setkey(cirs2, dists, indices)
@@ -583,11 +677,13 @@ test_that("distanceFromPoints does not work correctly", {
   })
   dfep <- distanceFromEachPoint(coords[, c("x", "y"), drop = FALSE],
                                 landscape = hab, cumulativeFn = `+`)
-  expect_true(sum(idw - dfep[, "val"]) %==% 0)
+  expect_true(sum(idw - dfep[, "dists"]) %==% 0)
 })
 
 test_that("simple cir does not work correctly", {
   set.seed(1234)
+  # dqrng::dqset.seed(1234)
+
   library(fpCompare)
   library(raster)
 
@@ -751,11 +847,11 @@ test_that("multi-core version of distanceFromEachPoints does not work correctly"
       clusterEvalQ(cl1, {
         library(SpaDES.tools)})
     })
-    system.time(
+    system.time({
       dfepCluster <- distanceFromEachPoint(coords[, c("x", "y"), drop = FALSE],
                                            landscape = hab, cumulativeFn = `+`,
                                            cl = cl1)
-    )
+    })
     stopCluster(cl1)
     expect_true(all.equal(dfep, dfepCluster))
 
@@ -763,10 +859,10 @@ test_that("multi-core version of distanceFromEachPoints does not work correctly"
     system.time({
       beginCluster(1)
     })
-    system.time(
+    system.time({
       dfepCluster2 <- distanceFromEachPoint(coords[, c("x", "y"), drop = FALSE],
                                             landscape = hab, cumulativeFn = `+`)
-    )
+    })
     endCluster()
     expect_true(all.equal(dfep, dfepCluster2))
   }
@@ -781,6 +877,7 @@ test_that("spreadProb with relative values does not work correctly", {
 
   seed <- 64350
   set.seed(seed)
+  # dqrng::dqset.seed(seed)
   emptyRas <- raster(extent(0, 1e2, 0, 1e2), res = 1)
   hab <- randomPolygons(emptyRas, numTypes = 40)
   names(hab) <- "hab"
@@ -788,22 +885,25 @@ test_that("spreadProb with relative values does not work correctly", {
   hab3 <- (hab > 20) * 200 + 1
   sam <- sample(which(hab3[] == 1), 1)
   set.seed(seed)
+  # dqrng::dqset.seed(seed)
+
   events1 <- spread(hab3, spreadProb = hab3, loci = sam, directions = 8,
                     neighProbs = c(0, 1), maxSize = c(100), exactSizes = TRUE)
 
   # Compare to absolute probability version
   set.seed(seed)
+  # dqrng::dqset.seed(seed)
+
   events2 <- spread(hab3, id = TRUE, loci = sam, directions = 8,
                     neighProbs = c(0, 1), maxSize = c(100), exactSizes = TRUE)
 
-  if (as.numeric_version(paste0(R.version$major, ".", R.version$minor)) < "3.6.0") {
+  #if (as.numeric_version(paste0(R.version$major, ".", R.version$minor)) < "3.6.4") {
     ## many more high value hab pixels spread to in event1
-    expect_true(sum(hab3[events1[] > 0]) > sum(hab3[events2[] > 0]))
-  } else {
+  #  expect_true(sum(hab3[events1[] > 0]) > sum(hab3[events2[] > 0]))
+  #} else {
     ## equal number on R-devel
     expect_equal(sum(hab3[events1[] > 0]), sum(hab3[events2[] > 0]))
-  }
-
+  #}
 
   # Check numeric vector with NAs is equivalent to raster with NAs
   numNAs <- 50
@@ -811,10 +911,12 @@ test_that("spreadProb with relative values does not work correctly", {
   ras <- raster(hab3)
   ras[] <- sps
   set.seed(seed)
+  # dqrng::dqset.seed(seed)
+
   expect_that(out1 <- spread(hab3, loci = ncell(hab3) / 2, spreadProb = ras), is_a("RasterLayer"))
   set.seed(seed)
+  # dqrng::dqset.seed(seed)
+
   expect_that(out2 <- spread(hab3, loci = ncell(hab3) / 2, spreadProb = sps), is_a("RasterLayer"))
   expect_identical(out1, out2)
-
 })
-
